//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __InjEct__
#define __InjEct__

#include "Streams.h"
#include "InjRea.h"
#include "GwollumPlot.h"

using namespace std;

/**
 * Inject signals in a data stream.
 * \author    Florent Robinet
 */
class InjEct: public InjRea, public GwollumPlot {
  
 public:
  
  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the InjEct class.
   * Injections listed in the ROOT files designated by a file pattern are loaded. It is important that the injection files were generated using the InjGen class. Injection files in the pattern must be time-sorted and with no overlap.
   *
   * The input stream, in which injections will be made, must be a LV detector and have a native sampling frequency larger than 16Hz.
   * @param aStream input data stream in which to inject
   * @param aPattern injection file pattern
   * @param aVerbose verbosity level
   */
  InjEct(Streams* aStream, const string aPattern, const int aVerbose=0);

  /**
   * Destructor of the InjEct class.
   */
  virtual ~InjEct();
  /**
     @}
  */

  /**
   * Updates the injection sampling frequency.
   * Use this function whenever the native sampling frequency of the input stream is changed. Indeed, the size of injection containers need to be updated.
   */
  void UpdateNativeSamplingFrequency(void);
 
  /**
   * Injects in a data vector.
   * @param aDataSize input vector size
   * @param aData pointer to input data vector (time-domain)
   * @param aTimeStart GPS time of first sample
   */
  bool Inject(const int aDataSize, double *aData, const double aTimeStart);
 
  /**
   * Plots a given injection waveforms.
   * 3 waveforms are plotted:
   * - h+ (geocentric time)
   * - hx (geocentric time)
   * - hdet projected in the detector (local time)
   * @param aInjIndex injection index
   * 
   */
  bool PlotInjection(const int aInjIndex);

 private:

  Streams *InStream;

  // INJECTION
  int sampling;             ///< injection sampling frequency
  double *window;           ///< tukey window
  TGraph *hplus;            ///< injection waveform (+)
  TGraph *hcross;           ///< injection waveform (x)
  TGraph *hdet;             ///< injection waveform projected on the detector
    
  bool GenerateWaveforms(const int aInjIndex);    ///< generate waveforms h+/hx/hdet
  void GenerateSineGauss();    ///< generate Sinegauss h+/hx
  void Project(void);                             ///< project waveform onto the detector
  double* GetTukeyWindow(const int aSize, 
			 const int aFractionSize);///< generate tukey window

  ClassDef(InjEct,0)
};

#endif


