//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __TriggerSelect__
#define __TriggerSelect__

#include "ReadTriggers.h"
#include "TH1.h"
#include "TH2.h"

using namespace std;

/**
 * Select trigger parameters.
 * This class was designed to operate a selection over trigger parameters: time, frequency, Q and SNR. For each parameter, the user can select a minimum, a maximum and a resolution. The resolution is the number used to bin a parameter range. The time is linearly binned while the other parameters are logarithmically binned. Once a selection is defined, pre-binned histograms can be extracted.
 * \author Florent Robinet
 */
class TriggerSelect{
  
 public:
  
  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the TriggerSelect class. 
   * The default selection is:
   * - Time:      700,000,000 --> 2,000,000,000 (1 bins) 
   * - Frequency:           1 --> 10000         (1 bins)
   * - Q:                   1 --> 1000          (1 bins)
   * - SNR:                 1 --> 1e+20         (1 bins)
   * @param aVerbose verbosity level
   */
  TriggerSelect(const int aVerbose=0);

  /**
   * Constructor of the TriggerSelect class.
   * The default selection is defined using the metadata (min/max) of the input ReadTriggers object. For each parameter the resolution is set to 1.
   * @param aTriggers input triggers
   * @param aVerbose verbosity level
   */
  TriggerSelect(ReadTriggers *aTriggers, const int aVerbose=0);

  /**
   * Destructor of the TriggerSelect class.
   */
  virtual ~TriggerSelect(void);
  /**
     @}
  */

  /**
   * Resets all the selection back to default.
   */
  void ResetSelection(void);

  /**
   * Sets the time selection.
   * @param aResolution resolution
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetTimeSelection(const int aResolution, const double aMin, const double aMax);

  /**
   * Sets the time resolution.
   * @param aResolution resolution
   */
  void SetTimeResolution(const int aResolution);

  /**
   * Sets the minimal time value.
   * @param aMin minimal value
   */
  void SetTimeMin(const double aMin);

  /**
   * Sets the maximal time value.
   * @param aMax maximal value
   */
  void SetTimeMax(const double aMax);

  /**
   * Sets the time range.
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetTimeRange(const double aMin, const double aMax);

  /**
   * Sets the frequency selection.
   * @param aResolution resolution
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetFrequencySelection(const int aResolution, const double aMin, const double aMax);

  /**
   * Sets the frequency resolution.
   * @param aResolution resolution
   */
  void SetFrequencyResolution(const int aResolution);

  /**
   * Sets the minimal frequency value.
   * @param aMin minimal value
   */
  void SetFrequencyMin(const double aMin);

  /**
   * Sets the maximal frequency value.
   * @param aMax maximal value
   */
  void SetFrequencyMax(const double aMax);

  /**
   * Sets the frequency range.
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetFrequencyRange(const double aMin, const double aMax);

  /**
   * Sets the Q selection.
   * @param aResolution resolution
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetQSelection(const int aResolution, const double aMin, const double aMax);

  /**
   * Sets the Q resolution.
   * @param aResolution resolution
   */
  void SetQResolution(const int aResolution);

  /**
   * Sets the minimal Q value.
   * @param aMin minimal value
   */
  void SetQMin(const double aMin);

  /**
   * Sets the maximal Q value.
   * @param aMax maximal value
   */
  void SetQMax(const double aMax);

  /**
   * Sets the Q range.
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetQRange(const double aMin, const double aMax);

  /**
   * Sets the SNR selection.
   * @param aResolution resolution
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetSNRSelection(const int aResolution, const double aMin, const double aMax);

  /**
   * Sets the SNR resolution.
   * @param aResolution resolution
   */
  void SetSNRResolution(const int aResolution);

  /**
   * Sets the minimal SNR value.
   * @param aMin minimal value
   */
  void SetSNRMin(const double aMin);

  /**
   * Sets the maximal SNR value.
   * @param aMax maximal value
   */
  void SetSNRMax(const double aMax);

  /**
   * Sets the SNR range.
   * @param aMin minimal value
   * @param aMax maximal value
   */
  void SetSNRRange(const double aMin, const double aMax);


  /**
   * Returns the minimal time value.
   */
  inline double GetTimeMin(void){ return tmin; };

  /**
   * Returns the maximal time value.
   */
  inline double GetTimeMax(void){ return tmax; };

  /**
   * Returns the time range value.
   */
  inline double GetTimeRange(void){ return tmax-tmin; };

  /**
   * Returns the time resolution value.
   */
  inline int GetTimeResolution(void){ return ntbins; };

  /**
   * Returns the minimal frequency value.
   */
  inline double GetFrequencyMin(void){ return fmin; };

  /**
   * Returns the maximal frequency value.
   */
  inline double GetFrequencyMax(void){ return fmax; };
 
  /**
   * Returns the frequency range value.
   */
  inline double GetFrequencyRange(void){ return fmax-fmin; };

  /**
   * Returns the frequency resolution value.
   */
  inline int GetFrequencyResolution(void){ return nfbins; };

  /**
   * Returns the minimal Q value.
   */
  inline double GetQMin(void){ return qmin; };

  /**
   * Returns the maximal Q value.
   */
  inline double GetQMax(void){ return qmax; };

  /**
   * Returns the Q range value.
   */
  inline double GetQRange(void){ return qmax-qmin; };

  /**
   * Returns the Q resolution value.
   */
  inline int GetQResolution(void){ return nqbins; };

  /**
   * Returns the minimal SNR value.
   */
  inline double GetSNRMin(void){ return smin; };

  /**
   * Returns the maximal SNR value.
   */
  inline double GetSNRMax(void){ return smax; };

  /**
   * Returns the SNR range value.
   */
  inline double GetSNRRange(void){ return smax-smin; };

  /**
   * Returns the SNR resolution value.
   */
  inline int GetSNRResolution(void){ return nsbins; };


  /**
   * Returns an empty 1-D histogram binned in a given parameter.
   * 'aParameter' can either be \"Time\", \"Frequency\", \"Q\" or \"SNR\". 'aParameter' is also used to name and title the histogram.
   *
   * NULL is returned if this function fails.
   * @param aParameter parameter name in which histograming
   */
  TH1D* GetTH1D(const string aParameter);

  /**
   * Returns an empty 2-D histogram binned in a given combination of parameters.
   * 'aParameterX/Y' can either be \"Time\", \"Frequency\", \"Q\" or \"SNR\". 'aParameterX/Y' are also used to name and title the histogram.
   *
   * NULL is returned if this function fails.
   * @param aParameterX parameter name in which histograming along the X axis
   * @param aParameterY parameter name in which histograming along the Y axis
   */
  TH2D* GetTH2D(const string aParameterX, const string aParameterY);

  /**
   * Prints the current selection in the standard output.
   */ 
  void PrintSelection(void);

 private:

  int Verbose;               ///< verbosity level

  // TIME
  void MakeTbins(void);      ///< make time binning (linear)
  double tmin, tmin_0;       ///< time min current/default
  double tmax, tmax_0;       ///< time max current/default
  int ntbins, ntbins_0;      ///< number of time bins current/default
  double *tbins;             ///< time bins

  // FREQUENCY
  void MakeFbins(void);      ///< make frequency binning (log)
  double fmin, fmin_0;       ///< frequency min current/default
  double fmax, fmax_0;       ///< frequency max current/default
  int nfbins, nfbins_0;      ///< number of frequency bins current/default
  double *fbins;             ///< frequency bins

  // Q
  void MakeQbins(void);      ///< make q binning (log)
  double qmin, qmin_0;       ///< q min current/default
  double qmax, qmax_0;       ///< q max current/default
  int nqbins, nqbins_0;      ///< number of q bins current/default
  double *qbins;             ///< q bins

  // SNR
  void MakeSbins(void);      ///< make q binning (log)
  double smin, smin_0;       ///< q min current/default
  double smax, smax_0;       ///< q max current/default
  int nsbins, nsbins_0;      ///< number of q bins current/default
  double *sbins;             ///< q bins

  ClassDef(TriggerSelect,0)  
};

#endif


